<!-- -*- mode: markdown; mode: flyspell -*- -->

# bash-base library

A library that provides a set of base functionality like logging and
error handling. To use it just source it at the very beginning of your
bash script like that:

```bash
source /usr/libexec/bash-base/base.bash || {
    echo "$0: fatal error: failed to source base.bash"
    exit 1
}
```

## API Docu

### Bash Base Version

```bash
# version
# returns the version of bash base
```

### Error Handling

```bash
# fatal <message> [..]
# end program execution after logging message prepended with 'fatal error'
```

```bash
# bb_traps_setup
# sets up traps for ERR and EXIT to handle unexpected errors and exits
```

```bash
# bb_traps_disable
# disables traps for ERR and EXIT, must be called before clean shutdown
```

### Logging

```bash
# log <message> [..]
# logs message to stderr
```

```bash
# log_debug <level> <message> [..]
# logs message if $LOG_LEVEL is set and it is level <= $LOG_LEVEL
```

## Example Code

### Hello World Script

```bash
#!/bin/bash

# source bash base library
source /usr/libexec/bash-base/base.bash || {
	echo "$0: fatal error: failed to source /usr/libexec/bash-base/base.bash"
	exit 1
}

# sets up traps used by bash-base to detect runtime errors
bb_traps_setup

# always put your code into some function, e.g. main
function main() {
	log "hello world"
}

main

# always unset traps before you shutdown
bb_traps_disable
exit 0
```

### Various Error Cases

```bash
#!/bin/bash

source /usr/libexec/bash-base/base.bash || {
	echo "$0: fatal error: failed to source /usr/libexec/bash-base/base.bash"
	exit 1
}

# sets up traps used by bash-base to detect runtime errors
bb_traps_setup


# replace the contents with your code
function main() {
    echo "this function is an example"
    echo "(un)comment lines below to test what happens on various failures"

    echo "main was called with args: $*"

    ## fatal error manually stops execution whereever called
    ## still you will get stack trace so you know where it happened
    fatal "kenny?"

    ## access to unset variables will trigger an error that is not catched
    ## by the ERR trap but by the EXIT trap. As a result the last line
    ## number of the stack trace will be incorrect (always 1)
    echo "$NoSuchVariableDefined"

    ## a command that fails: will trigger an error inside of main
    false

    ## this will sucessfully end function main, but since the return value
    ## is != 0 it will trigger an error where main was called
    return 123
}

# just call a function, never run code directly
main "$@"

# unset traps before you shutdown
bb_traps_disable

echo "$0: clean shutdown"
exit 0
```
